body {
    background-color: #eeeeec;
    font-size: 100%; line-height: 1.25; text-align: center; }

.container {
    background-color: #fff;
    width: 60.5em; min-height: 57.25em;
    text-align: left; margin: 0 auto; }

.content {
    margin-bottom: 5em;
    text-align: justify; display: inline-block; }

.document {
    float: left;
    width: 36.8em; margin-right: 2.5em; }

.document a, .footer a {
    text-decoration: underline; }

.sidebar {
    float: left; width: 18em;
    text-align: left; }

.related { overflow: hidden; }

.related h3 {
    font-size: 1px; text-indent: -9999999em; }

.related ul { margin: 0; }

.related li {
    display: inline; list-style-type: none; }

.rel-main { float: left; }
.rel-extra { float: right; }

.header,
.related,
.content,
.footer {
    margin-left: 1.5em; margin-right: 1.5em; }

.header {
    border-bottom: 3px solid #2e3436; }

.header h1 { margin: 0; }

.footer {
    padding-top: 2em; text-align: right;
    border-top: 4px solid #babdb6; }

/*
 * http://lamb.cc/typograph/
 */

h1, h2, h3, h4 {
    color: #3465a4; font-family: georgia; font-weight: normal; }

h1 { font-size: 1.75em; margin: 1.51786em 0; color: #204a87; }
h2 {
    border-bottom: 1px solid #3465a4;
    font-size: 1.5em; margin: 1.45833em 0; }
h3 { font-size: 1.33333em; margin: 1.25em 0; }
h4 { font-size: 1.16667em; margin: 1.51786em 0; }

a {
    color: #ce5c00; }

.headerlink {
    visibility: hidden; color: #dddddd; padding-left: .3em; }

h1:hover > .headerlink,
h2:hover > .headerlink,
h3:hover > .headerlink,
h4:hover > .headerlink,
h5:hover > .headerlink,
h6:hover > .headerlink,
dt:hover > .headerlink { visibility: visible; }

img {
    border: 0; }

dt:target, .highlighted {
    background-color: #fbe54e; }

/*
 * http://www.blueprintcss.org/
 */

.small     { font-size: .8em; margin-bottom: 1.875em; line-height: 1.875em; }
.large     { font-size: 1.2em; line-height: 2.5em; margin-bottom: 1.25em; }
.hide      { display: none; }

.quiet     { color: #666; }
.loud      { color: #000; }
.highlight { background: #ff0; }
.added     { background: #060; color: #fff; }
.removed   { background: #900; color: #fff; }

.first     { margin-left: 0; padding-left: 0; }
.last      { margin-right: 0; padding-right: 0; }
.top       { margin-top: 0; padding-top: 0; }
.bottom    { margin-bottom: 0; padding-bottom: 0; }

/*
 *
 */

.doc-link,
.feat-link {
    margin-left: 0; text-align: left; }

.doc-link dd,
.feat-link dd {
    margin-left: 0; font-style: italic; }

.feat-link dt {
    color: #3465a4; font-weight: bold; }

.doc-link {
    width: 44%; height: 6em; float: left;
    margin-right: 2em; }

/*
 *
 */

.sidebar .toctree-l1.current a {
    border-right: 5px solid #fcaf3e; }

.line-block {
    display: block;
    margin-top: 1em;
    margin-bottom: 1em;
}

.line-block .line-block {
    margin-top: 0;
    margin-bottom: 0;
    margin-left: 1.5em;
}

div.header div.rel a {
  color:  #fcaf3e;
  letter-spacing: .1em;
  text-transform: uppercase;
}

.highlight {
  padding: 3px;
  background-color: #eeeeec;
  border-top: 2px solid #dddddd;
  border-bottom: 2px solid #dddddd;
  margin-top: .8em;
  margin-bottom: .8em;
}

.descname {
  font-weight: bold; }

.literal {
  background-color: #eeeeec; }

blockquote {
  margin: 1em; }

.footer, .footer a {
  color: #888a85; }

div.admonition {
    font-size: 0.9em;
    margin: 1em 0 1em 0;
    padding: 0.5em 1em 0.5em 1em;
    border: 1px solid #ddd; }

div.admonition p.admonition-title {
    font-weight: bold; color: #3465a4; }

div.warning {
    border-color: #940000; }

div.warning p.admonition-title {
    color: #940000; }

div.viewcode-block:target {
    background-color: #f4debf;
    border-top: 1px solid #ac9;
    border-bottom: 1px solid #ac9;
}

/* Styles copied from basic theme */

img.align-left, .figure.align-left, object.align-left {
    clear: left; float: left; margin-right: 1em; }

img.align-right, .figure.align-right, object.align-right {
    clear: right; float: right; margin-left: 1em; }

img.align-center, .figure.align-center, object.align-center {
  display: block; margin-left: auto; margin-right: auto; }

.align-left { text-align: left; }
.align-center { text-align: center; }
.align-right { text-align: right; }

ul.keywordmatches li.goodmatch a {
    font-weight: bold; }

/* -- index page ------------------------------------------------------------ */

table.contentstable {
    width: 90%;
}

table.contentstable p.biglink {
    line-height: 150%;
}

a.biglink {
    font-size: 1.3em;
}

span.linkdescr {
    font-style: italic;
    padding-top: 5px;
    font-size: 90%;
}

/* -- general index --------------------------------------------------------- */

table.indextable td {
    text-align: left;
    vertical-align: top;
}

table.indextable dl, table.indextable dd {
    margin-top: 0;
    margin-bottom: 0;
}

table.indextable tr.pcap {
    height: 10px;
}

table.indextable tr.cap {
    margin-top: 10px;
    background-color: #f2f2f2;
}

img.toggler {
    margin-right: 3px;
    margin-top: 3px;
    cursor: pointer;
}

/* -- viewcode extension ---------------------------------------------------- */

.viewcode-link {
    float: right;
}

.viewcode-back {
    float: right;
}

div.viewcode-block:target {
    margin: -1px -3px;
    padding: 0 3px;
    background-color: #f4debf;
    border-top: 1px solid #ac9;
    border-bottom: 1px solid #ac9;
}
